#include <cstdio>
#include <iostream>
#include <sstream>
#include "ARToolKitPlus/TrackerSingleMarker.h"

#include "com_studierstube_ARTKP_ARTKP.h"
#include <jni.h>

#include "marker_cfg.h"

#include "AR/nyar_NyARTransMat.h"

const char camera_cfg[] = "ARToolKitPlus_CamCal_Rev02\n640 480 129.69 106.73 246.19 246.63 -0.4186 0.1791 -0.0002 0.0026 0.0000 0.0000 10\n";

const char camera_calib[] = {
    0x00, 0x00, 0x02,-0x80, 0x00, 0x00, 0x01,-0x20,
    0x40,-0x7B,-0x19,-0x64,-0x64, 0x75,-0x2E, 0x47,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x40, 0x73,-0x38, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x40,-0x7A,-0x50,-0x40,-0x1E, 0x4F,-0x72,-0x6A,
    0x40, 0x6E, 0x30, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x3F,-0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x40, 0x73,-0x18, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x40, 0x70, 0x78, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x40, 0x3A, 0x33, 0x33, 0x33, 0x33, 0x33, 0x33,
    0x3F,-0x10, 0x34, 0x40, 0x2E, 0x71, 0x21, 0x0C
};

using ARToolKitPlus::TrackerSingleMarker;
using namespace std;

// create a tracker that does:
//  - 6x6 sized marker images (required for binary markers)
//  - samples at a maximum of 6x6
//  - works with luminance (gray) images
//  - can load a maximum of 0 non-binary pattern
//  - can detect a maximum of 8 patterns in one image
static TrackerSingleMarker* tracker;

static bool inited = false;

static jbyte* img;

static nyar_NyARTransMat_O2_t* nya;

void init(int width, int height)
{
    img = new jbyte[width * height];
    
    nya=nyar_NyARTransMat_O2_create(&p2);
    
    tracker = new TrackerSingleMarker(width, height, 8, 6, 6, 6, 0);

    tracker->setPixelFormat(ARToolKitPlus::PIXEL_FORMAT_LUM);

    istringstream ccfg(camera_cfg);
    //istringstream mcfg(marker_cfg);

    // load a camera file.
    if (!tracker->init(ccfg, 1.0f, 1000.0f)) {
        printf("ERROR: init() failed\n");
        return;
    }

    // the marker in the BCH test image has a thiner border...
    tracker->setBorderWidth(0.25f);

    // activate automatic thresholding
    tracker->activateAutoThreshold(true);

    // let's use lookup-table undistortion for high-speed
    // note: LUT only works with images up to 1024x1024
    tracker->setUndistortionMode(ARToolKitPlus::UNDIST_LUT);

    // switch to simple ID based markers
    // use the tool in tools/IdPatGen to generate markers
    tracker->setMarkerMode(ARToolKitPlus::MARKER_ID_SIMPLE);
    
    tracker->setImageProcessingMode(ARToolKitPlus::IMAGE_HALF_RES);
    
    /*tracker->setHullMode(ARToolKitPlus::HULL_OFF);
    tracker->setPoseEstimator(ARToolKitPlus::POSE_ESTIMATOR_ORIGINAL_CONT);*/
}

JNIEXPORT jfloatArray JNICALL Java_com_studierstube_ARTKP_ARTKP_getProjectionMatrix
  (JNIEnv *env, jclass clazz)
{
    jfloatArray flarr = env->NewFloatArray(16);
    env->SetFloatArrayRegion(flarr, 0, 16, tracker->getProjectionMatrix());
    return flarr;
}

JNIEXPORT jfloatArray JNICALL Java_com_studierstube_ARTKP_ARTKP_getModelViewMatrix
  (JNIEnv *env, jclass clazz)
{
    jfloatArray flarr = env->NewFloatArray(16);
    for (int i = 0; i < 16; i++)
        printf("%.2f  %s", tracker->getModelViewMatrix()[i], (i % 4 == 3) ? "\n  " : "");
    fflush(stdout);
    env->SetFloatArrayRegion(flarr, 0, 16, tracker->getModelViewMatrix());
    return flarr;
}

JNIEXPORT jint JNICALL Java_com_studierstube_ARTKP_ARTKP_process
  (JNIEnv* env, jclass clazz, jbyteArray data, jint width, jint height)
{
    const int bpp = 1;
    
    if (!inited)
    {
        init(width, height);
        inited = true;
    }
    
    env->GetByteArrayRegion(data, 0, width * height, img);
    
    ARMarkerInfo* markers = NULL;
    int markerCount = 0;

    // here we go, just one call to find the camera pose
    std::vector<int> markerId = tracker->calc((unsigned char*)img, &markers, &markerCount);
    /*//int id = tracker->selectBestMarkerByCf();
    tracker->selectDetectedMarker(488);
    float conf = tracker->getConfidence();
    printf("conf: %f\n", conf);

    return 488; //id;*/
    
    nyar_NyARTransMat_O2_transMat(nya, marker_info, gPatt_centre, gPatt_width, gPatt_trans);
    
    
    /*int count = tracker->calc((unsigned char*)img);

    return count;*/
}

/*
xstat.clear(); ystat.clear();
for( int i = 0; i < points0.size(); i++ )
{
    if( !status[i] )
        continue;
    
    Point2f pt0 = points0[i];
    Point2f pt1 = points1[i];
    
    xstat.push_back(pt1.x - pt0.x);
    ystat.push_back(pt1.y - pt0.y);
    
    //circle(frame, pt1, 1, Scalar(0,255,0), -1);
}

// should replace these with select instead of sort
sort(xstat.begin(), xstat.end());
sort(ystat.begin(), ystat.end());
dx = xstat[xstat.size()/2];
dy = ystat[ystat.size()/2];
*/
