package edu.cmu.ece.drawe3.render;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import android.opengl.GLSurfaceView;

public class CubeRenderer implements GLSurfaceView.Renderer {
	
	float[] modeltrans;
	float[] projection;
	
	private boolean resetProj = false;
	
	public CubeRenderer()
	{
		modeltrans = null;
		projection = null;
	}
	
	public void setProjection(float[] _proj) {
		this.projection = _proj;
		
		this.resetProj = true;
	}

	public void setModelView(float[] _trans)
	{
		this.modeltrans = _trans;
	}

    public void onDrawFrame(GL10 gl) {
        /*
         * Usually, the first thing one might want to do is to clear
         * the screen. The most efficient way of doing this is to use
         * glClear().
         */

        gl.glClear(GL10.GL_COLOR_BUFFER_BIT | GL10.GL_DEPTH_BUFFER_BIT);

        /*
         * Now we're ready to draw some 3D objects
         */
        
        if (resetProj)
        {
        	gl.glMatrixMode(GL10.GL_PROJECTION);
            gl.glLoadIdentity();
            if (this.projection != null)
            	gl.glLoadMatrixf(projection, 0);
            this.resetProj = false;
        }

        gl.glMatrixMode(GL10.GL_MODELVIEW);
        gl.glLoadIdentity();
        if (this.modeltrans != null)
        	gl.glLoadMatrixf(this.modeltrans, 0);

        gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
        gl.glEnableClientState(GL10.GL_COLOR_ARRAY);

        if (this.modeltrans != null)
        	mCube.draw(gl);
    }

    public void onSurfaceChanged(GL10 gl, int width, int height) {
         gl.glViewport(0, 0, width, height);

         /*
          * Set our projection matrix. This doesn't have to be done
          * each time we draw, but usually a new projection needs to
          * be set when the viewport is resized.
          */

         //float ratio = (float) width / height;
         gl.glMatrixMode(GL10.GL_PROJECTION);
         //gl.glLoadIdentity();
         //gl.glFrustumf(-ratio, ratio, -1, 1, 1, 10);
         if (this.projection != null)
        	 gl.glLoadMatrixf(projection, 0);
    }

    public void onSurfaceCreated(GL10 gl, EGLConfig config) {
        /*
         * By default, OpenGL enables features that improve quality
         * but reduce performance. One might want to tweak that
         * especially on software renderer.
         */
        gl.glDisable(GL10.GL_DITHER);

        /*
         * Some one-time OpenGL initialization can be made here
         * probably based on features of this particular context
         */
         gl.glHint(GL10.GL_PERSPECTIVE_CORRECTION_HINT,
                 GL10.GL_FASTEST);

         gl.glClearColor(0,0,0,0);
         
         gl.glEnable(GL10.GL_CULL_FACE);
         gl.glShadeModel(GL10.GL_SMOOTH);
         gl.glEnable(GL10.GL_DEPTH_TEST);
    }

    private Cube mCube = new Cube();
}

class Cube
{
    public Cube()
    {
        float vertices[] = {
                -50.0f, -50.0f, -50.0f,
                50.0f, -50.0f, -50.0f,
                50.0f,  50.0f, -50.0f,
                -50.0f,  50.0f, -50.0f,
                -50.0f, -50.0f,  50.0f,
                50.0f, -50.0f,  50.0f,
                50.0f,  50.0f,  50.0f,
                -50.0f,  50.0f,  50.0f,
        };

        float colors[] = {
                0,    0,    0,  1.0f,
                1.0f,    0,    0,  1.0f,
                1.0f,  1.0f,    0,  1.0f,
                0,  1.0f,    0,  1.0f,
                0,    0,  1.0f,  1.0f,
                1.0f,    0,  1.0f,  1.0f,
                1.0f,  1.0f,  1.0f,  1.0f,
                0,  1.0f,  1.0f,  1.0f,
        };

        byte indices[] = {
                0, 4, 5,    0, 5, 1,
                1, 5, 6,    1, 6, 2,
                2, 6, 7,    2, 7, 3,
                3, 7, 4,    3, 4, 0,
                4, 7, 6,    4, 6, 5,
                3, 0, 1,    3, 1, 2
        };

        // Buffers to be passed to gl*Pointer() functions
        // must be direct, i.e., they must be placed on the
        // native heap where the garbage collector cannot
        // move them.
        //
        // Buffers with multi-byte datatypes (e.g., short, int, float)
        // must have their byte order set to native order

        ByteBuffer vbb = ByteBuffer.allocateDirect(vertices.length*4);
        vbb.order(ByteOrder.nativeOrder());
        mVertexBuffer = vbb.asFloatBuffer();
        mVertexBuffer.put(vertices);
        mVertexBuffer.position(0);

        ByteBuffer cbb = ByteBuffer.allocateDirect(colors.length*4);
        cbb.order(ByteOrder.nativeOrder());
        mColorBuffer = cbb.asFloatBuffer();
        mColorBuffer.put(colors);
        mColorBuffer.position(0);

        mIndexBuffer = ByteBuffer.allocateDirect(indices.length);
        mIndexBuffer.put(indices);
        mIndexBuffer.position(0);
    }

    public void draw(GL10 gl)
    {
        gl.glFrontFace(GL10.GL_CW);
        gl.glVertexPointer(3, GL10.GL_FLOAT, 0, mVertexBuffer);
        gl.glColorPointer(4, GL10.GL_FLOAT, 0, mColorBuffer);
        gl.glDrawElements(GL10.GL_TRIANGLES, 36, GL10.GL_UNSIGNED_BYTE, mIndexBuffer);
    }

    private FloatBuffer   mVertexBuffer;
    private FloatBuffer   mColorBuffer;
    private ByteBuffer  mIndexBuffer;
}