package edu.cmu.ece.drawe3.tracking;

import java.io.IOException;

import android.app.Activity;
import android.hardware.Camera;
import android.hardware.Camera.PreviewCallback;
import android.hardware.Camera.Size;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.ViewGroup;

public class CameraSource extends SurfaceView implements SurfaceHolder.Callback {

	private Camera mCamera;
	private SurfaceHolder mHolder;
    private PreviewCallback pc = null;
	
	public CameraSource(Activity context) {
		super(context);
		
		context.addContentView(this, new ViewGroup.LayoutParams(ViewGroup.LayoutParams.FILL_PARENT,
                ViewGroup.LayoutParams.FILL_PARENT));
        
        // Install a SurfaceHolder.Callback so we get notified when the
        // underlying surface is created and destroyed.
		this.mCamera = Camera.open();
        this.mHolder = getHolder();
        this.mHolder.addCallback(this);
        this.mHolder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);
	}
	
	public Size getSize() {
		if (mCamera != null)
			return mCamera.getParameters().getPictureSize();
		else
			return null;
	}

    public void surfaceCreated(SurfaceHolder holder) {
        // The Surface has been created, acquire the camera and tell it where
        // to draw.
    	if (mCamera == null)
    	{
    		this.mCamera = Camera.open();
    	}
    	
        try {
           mCamera.setPreviewDisplay(holder);
           if (pc != null)
              mCamera.setPreviewCallback(pc);
        } catch (IOException exception) {
            mCamera.release();
            mCamera = null;
        }
    }

    public void surfaceDestroyed(SurfaceHolder holder) {
        // Surface will be destroyed when we return, so stop the preview.
        // Because the CameraDevice object is not a shared resource, it's very
        // important to release it when the activity is paused.
    	
	    this.release();
    }

    public void surfaceChanged(SurfaceHolder holder, int format, int w, int h) {
        // Now that the size is known, set up the camera parameters and begin
        // the preview.
    	if (mCamera != null)
    	{
    		Camera.Parameters parameters = mCamera.getParameters();
    		if(w>h){
    			parameters.set("orientation", "landscape");
    			parameters.setRotation(0);
    		}
    		else{
    			parameters.set("orientation", "portrait");
    			parameters.setRotation(90);
    		}
	        /*parameters.setPreviewSize(w, h);*/
	        mCamera.setParameters(parameters);
	        mCamera.startPreview();
    	}
    }
    
    public void release()
    {
    	if (mCamera != null)
    	{
	    	mCamera.stopPreview();
	    	mCamera.setPreviewCallback(null);
	        mCamera.release();
	        mCamera = null;
    	}
    }

    public void setPreviewCallback(PreviewCallback pc)
    {
    	this.pc = pc;
    	if (mCamera != null)
    		mCamera.setPreviewCallback(pc);
    }
}