package edu.cmu.ece.drawe3.tracking.nyartk;

import java.io.ByteArrayInputStream;

import jp.nyatla.nyartoolkit.NyARException;
import jp.nyatla.nyartoolkit.core.param.NyARParam;
import jp.nyatla.nyartoolkit.jogl.utils.NyARGLUtil;
import android.hardware.Camera;

public class CameraCalibration
{
	private final static byte[] camera_calib = new byte[] {
		0x00, 0x00, 0x02, -0x80, 0x00, 0x00, 0x01, -0x20,
		0x40, -0x7B, -0x19, -0x64, -0x64, 0x75, -0x2E, 0x47,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x40, 0x73, -0x38, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x40, -0x7A, -0x50, -0x40, -0x1E, 0x4F, -0x72, -0x6A,
		0x40, 0x6E, 0x30, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x3F, -0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x40, 0x73, -0x18, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x40, 0x70, 0x78, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x40, 0x3A, 0x33, 0x33, 0x33, 0x33, 0x33, 0x33,
		0x3F, -0x10, 0x34, 0x40, 0x2E, 0x71, 0x21, 0x0C };
	
	public static NyARParam getCalibration(Camera.Size size)
	{
		NyARParam cparam= new NyARParam();
		try {
			cparam.loadARParam(new ByteArrayInputStream(camera_calib));
			cparam.changeScreenSize(size.width, size.height);
			return cparam;
		} catch (NyARException e) {
			e.printStackTrace();
			return null;
		}
	}
    
	public static float[] getProjection(Camera.Size size)
	{
        double[] dproj = new double[16];
        NyARGLUtil.toCameraFrustumRH(getCalibration(size),1.0,10,10000,dproj);
		float[] fproj = new float[16];
		for(int i = 0; i < 16; i++)
			fproj[i] = (float) dproj[i];
		return fproj;
	}
}