#ifndef COMMON_H_
#define COMMON_H_


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <setjmp.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <errno.h>
#include <math.h>
#include <pthread.h>
#include <semaphore.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>

/* Persistent state structure for our own socket IO (SIO) routines. */
typedef struct
{
	size_t num_unread;   /* Number of unread bytes */
	size_t unread_idx;   /* Index of the next unread byte */
	char   buffer[8192]; /* Internal buffer */
	int    fd;           /* File descriptor of the socket to read from */
}
sio_t;
	
/*
 * Error-handling functions.
 */
void print_posix_error(int code, const char *msg);
void print_unix_error(const char *msg);
void print_app_error(const char *msg);
int check_unix_error(int result, const char *msg);
int check_posix_error(int result, const char *msg);

int open_host_fd(const char *host, int port);
int get_host_addr(const char *name, struct in_addr *addr);

/*
 * Socket IO (SIO) functions.
 */
void sio_init_read(sio_t *sio, int fd); 
ssize_t sio_write(int fd, const void *buf, size_t n);
ssize_t	sio_read_helper(sio_t *sio, void *buf, size_t n);
ssize_t	sio_read(sio_t *sio, void *buf, size_t n);
ssize_t	sio_read_line(sio_t *sio, void *buf, size_t buf_size);


#endif
