//
//  SampleServer.java
//
//  Main server class. 
//  Will serve both android and kinect clients. 
//  TODO: Identify which client is using it. 
//  Added (2/28/11): Actually, there is no need for a KinectServerThread class. 
//      Both Kinect and Android can use the same serverThread, just need to listen on different ports.
//
//  Created by Yumin Wong on 2/25/11.
//  Last modified by Yumin Wong on 4/23/11.
//

//package edu.cmu.ece.drawe3.comm;

import java.io.*;
import java.net.*;
import java.util.*;
import java.util.concurrent.*;

// Server side
public class SampleServer
{
    private ConcurrentLinkedQueue<ClientThread> clients = new ConcurrentLinkedQueue<ClientThread>();
    private ConcurrentLinkedQueue<CoordPacket> coords = new ConcurrentLinkedQueue<CoordPacket>();
    
    private final int port;
    
    /**
     * Server that will listen at port <port>
     */
    public SampleServer(int port)
    {
        this.port = port;
    }
    
    class ClientThread extends Thread
    {
        public Socket s;
        
        public ClientThread(Socket s)
        {
            this.s = s;
        }
        
        public void run()
        {
            try
            {
                DataInputStream dsi = new DataInputStream(s.getInputStream());
                
                {
                    DataOutputStream dso = new DataOutputStream(s.getOutputStream());
                    
                    int i = 0;
                    for(CoordPacket cp : coords)
                    {
                        cp.toByteStream(dso);
                        i++;
                    }
                    dso.flush();
                    System.out.println("Sent " + i + " packet burst to " + s.getInetAddress().getHostAddress());
                }
                
                clients.add(this);
                
	            while(!s.isClosed())
	            {
	                CoordPacket cp = CoordPacket.fromByteStream(dsi);
	                coords.add(cp);
	                // for testing purposes.
	                System.out.println("Received packet from "+s.getInetAddress().getHostAddress());
	                System.out.println(cp.toString());
	                
	                System.out.println("Sending to " + (clients.size()-1) + " clients");
                    for(ClientThread c : clients)
                    {
                        if (c == this) continue;
                        
                        try {
                            DataOutputStream dso = new DataOutputStream(c.s.getOutputStream());
                            cp.toByteStream(dso);
                            dso.flush();
                        } catch (IOException e) {
                            clients.remove(c);
                            System.out.println("Removed client " + c.s.getInetAddress().getHostAddress() + " on failed write");
                        }
                    }
	            }
	            
	            System.out.println("Removed client " + s.getInetAddress().getHostAddress() + " on closed socket");
            } catch (IOException e) {
	            System.out.println("Removed client " + s.getInetAddress().getHostAddress() + " on failed read");
            } finally {
                clients.remove(this);
            }
        }
    }
    
    public void listen()
    {
        try
        {
            ServerSocket ss = new ServerSocket(port);
            
            while (!ss.isClosed()) {
                Socket s = ss.accept();
                
                ClientThread c = new ClientThread(s);
                c.start();

                System.out.println("Client has connected.");
                System.out.println(" Address is: "+s.getInetAddress().getHostAddress());
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
            System.out.println("Server terminated.");
        }
    }
       
    public static void main(String[] args)
    {
        // can create another serverSocket for android/kinect
        final SampleServer server = new SampleServer(1096);
        server.listen();
    }
}
